#include <hidef.h>      /* common defines and macros */
#include "derivative.h"      /* derivative-specific definitions */
//==============================================================================
// WARNING:
// In this code you have to ensure the code will never be executed out of 
// the EEPROM while EEEPROM is E/W
//==============================================================================
// Tested on TWR-S12G128
// // August/4/2014
//========================= GLOBAL DEFS ========================================
#define  TRUE   1
#define  FALSE  0
#define  OK     0
#define  ERROR  1

#define  UBYTE  unsigned char
#define  SBYTE           char
#define  UWORD  unsigned int
#define  SWORD           int
#define  ULONG  unsigned long
#define  SLONG           long

//========================= EEPROM SETUP DEFS ==================================
#define  EEPROM_START                       0x000400 
#define  EEPROM_END                         0x0013FF    
#define  EEPROM_SECTOR                      0x04

#define  EE_NOT_ALIGNED_ADDR                0xA3
#define  EE_NON_ERASED                      0xA4
#define  EE_ACCESS_ERROR                    0xA5
#define  EE_VERIFICATION_FAILED             0xA6
#define  EE_LENGTH_OUT_OF_RANGE             0xA7


//========================= GLOBAL VARS ========================================
UBYTE   CCRbackup; 
UWORD   buf[4];
UWORD   temp=0;

//========================= MACROS =============================================
#define KICKTHEDOG             { _FEED_COP(); }                           // refresh COP
#define getCCR_disI()          { asm TPA; asm STAA CCRbackup; asm SEI; }  // save CCR disable interrupts
#define restoreCCR()           { asm LDAA CCRbackup; asm TAP;  }          // restore CCR

//========================= FUNCTION PROETOTYPES ===============================
UBYTE EEPROM_Erase_Verify_Section(UWORD address, UWORD number_of_words);
UBYTE EEPROM_Program(UWORD address, UWORD *ptr, UBYTE number_of_words);
UBYTE EEPROM_Erase_Sector(UWORD address);
UWORD EEPROM_Read_Word(UWORD address);
void  EEPROM_Init(UBYTE fdiv);
UBYTE EEPROM_EraseAll(void);
//..............................................................................
void SetPEEmodeBUSCLK(UBYTE _synr, UBYTE _refdv, UBYTE _postdiv);
//..............................................................................
#pragma CODE_SEG NON_BANKED
interrupt 28 void PLL_LockIsr(void);
#pragma CODE_SEG DEFAULT
//==============================================================================
//==============================================================================
//EEPROM_Erase_Verify_Section
//==============================================================================
UBYTE EEPROM_Erase_Verify_Section(UWORD address, UWORD number_of_words)
{
  //check if address is aligned (global address [2:0] = 000)
  if((address & 0x0004) != 0)
    return EE_NOT_ALIGNED_ADDR; 
  
  while(FSTAT_CCIF == 0) KICKTHEDOG;   //wait if command in progress
  FSTAT = 0x30;                         //clear ACCERR and PVIOL
  
  FCCOBIX = 0x00;
  FCCOB   = 0x1000;
  
  FCCOBIX = 0x01;
  FCCOB   = address;
  
  FCCOBIX = 0x02;
  FCCOB   = number_of_words;
  
  getCCR_disI();
  FSTAT_CCIF = 1;         //launch command
  while(FSTAT_CCIF == 0); //wait for done
  restoreCCR();
  
  if((FSTAT & (FSTAT_ACCERR_MASK | FSTAT_FPVIOL_MASK)) != 0)
    return EE_ACCESS_ERROR;
  
  //check if phrases are erased and return result
  if(FSTAT_MGSTAT != 0)
    return EE_NON_ERASED;
  else
    return OK;
}
//==============================================================================
//EEPROM_Program
//==============================================================================
UBYTE EEPROM_Program(UWORD address, UWORD *ptr, UBYTE number_of_words)
{  
  UWORD i;
  
  if((number_of_words < 1) || (number_of_words > 4))
    return EE_LENGTH_OUT_OF_RANGE;
  
  //check if address is aligned (global address [0] != 0)
  if((address & 0x0001) != 0)
    return EE_NOT_ALIGNED_ADDR; 
  
  //check if the word(s) is/are erased    
  if((EEPROM_Erase_Verify_Section(address, number_of_words)) == EE_NON_ERASED)
    return EE_NON_ERASED;  
    
  while(FSTAT_CCIF == 0) KICKTHEDOG;   //wait if command in progress
  FSTAT = 0x30;             //clear ACCERR and PVIOL    
  
  FCCOBIX = 0x00;
  FCCOB   = 0x1100;
  
  FCCOBIX = 0x01;
  FCCOB   = address;
  
  
  for(i=1; i<=number_of_words; i++)  //fill appropriate number of words to FCCOB
  {
    FCCOBIX = i+1;
    FCCOB = *ptr;
    ptr++;  
  }

  getCCR_disI();
  FSTAT_CCIF = 1;         //launch command
  while(FSTAT_CCIF == 0); //wait for done
  restoreCCR();

  if((FSTAT & (FSTAT_ACCERR_MASK | FSTAT_FPVIOL_MASK)) != 0) return EE_ACCESS_ERROR;
  if(FSTAT_MGSTAT != 0) return EE_VERIFICATION_FAILED;    
  return OK;
}
//==============================================================================
//EEPROM_Erase_Sector
//==============================================================================
UBYTE EEPROM_Erase_Sector(UWORD address)
{
  //size of sector is 4B
  
  //check if address is aligned (global address [0] != 0)  
  if((address & 0x00000001) != 0)
    return EE_NOT_ALIGNED_ADDR; 
    
  while(FSTAT_CCIF == 0) KICKTHEDOG;   //wait if command in progress
  FSTAT = 0x30;             //clear ACCERR and PVIOL
  
  FCCOBIX = 0x00;
  FCCOB   = 0x1200;
  
  FCCOBIX = 0x01;
  FCCOB   = address;
   
  getCCR_disI();
  FSTAT_CCIF = 1;         //launch command
  while(FSTAT_CCIF == 0); //wait for done
  restoreCCR();
  
  if((FSTAT & (FSTAT_ACCERR_MASK | FSTAT_FPVIOL_MASK)) != 0) return EE_ACCESS_ERROR;
  if(FSTAT_MGSTAT != 0) return EE_VERIFICATION_FAILED;    
  return OK;
}
//==============================================================================
//EEPROM_Read_Word
//==============================================================================
UWORD EEPROM_Read_Word(UWORD address)
{
  UWORD data16;
  data16 = *(UWORD *)address;
  return data16;
}


//==============================================================================
//UBYTE EEPROM_EraseAll(void);
//==============================================================================
UBYTE EEPROM_EraseAll(void)
{
 UWORD addr;
 UBYTE err = OK;
 
 
 err = EEPROM_Erase_Verify_Section(EEPROM_START,512);
 if(err != EE_NON_ERASED) return(err);
 
 
 addr = EEPROM_START;
 do
    {
     //---------
     KICKTHEDOG;
     //---------
     err = EEPROM_Erase_Sector(addr);
     addr+=EEPROM_SECTOR;
    }
 while((!err) && (addr < EEPROM_END));
 
 err = EEPROM_Erase_Verify_Section(EEPROM_START,512);
 return(err);
}
//==============================================================================
//EEPROM_Init
//==============================================================================
void EEPROM_Init(UBYTE fdiv)
{  
  FCLKDIV = fdiv;           
}
//==============================================================================
//==============================================================================
//==============================================================================
// PLL Lock Isr
//==============================================================================
#pragma CODE_SEG NON_BANKED
interrupt 28 void PLL_LockIsr(void)
{
  if(CPMUFLG_LOCK == 0)  {    //do something here  
                         }
  else                   {    //do something here  
                         }
}
#pragma CODE_SEG DEFAULT
//==============================================================================
// void SetPEEmodeBUSCLK(UBYTE _synr, UBYTE _refdv, UBYTE _postdiv)
//==============================================================================
void SetPEEmodeBUSCLK(UBYTE _synr, UBYTE _refdv, UBYTE _postdiv)
{
 CPMUSYNR    = _synr;
 CPMUREFDIV  = _refdv;
 CPMUPOSTDIV = _postdiv;   
  
 CPMUOSC_OSCE = 1; //enable external oscillator OSCE


 //--------------------------------------
 //--- enable COP    --------------------
 //--------------------------------------
 CPMUCOP = 0x47;                          // period 2^24*oscclk=2.09s @ 8MHz OSCCLK, freeze COP in active BDM
 asm nop;
    
 while(!CPMUFLG_UPOSC)
  {// you can check for timeot here with error message report
  };    
 while(!CPMUFLG_LOCK)
  {// you can check for timeot here with error message report
  };    
 
 //--------------------------------------
 //--- select clocks --------------------
 //--------------------------------------
 CPMUCLKS = 0B10000001;                   // bus=fPLL/2; COP is clocked from OSCCLK

 if(CPMUCLKS != 0B10000001)               // After writing CPMUCLKS register, it is strongly recommended to read  
  {                                       // back CPMUCLKS register to make sure that write of PLLSEL,
     asm nop;                             // RTIOSCSEL, COPOSCSEL0 and COPOSCSEL1 was successful.
  }
 //--------------------------------------
}
//==============================================================================
//==============================================================================
// void main(void) 
//==============================================================================
//==============================================================================
void main(void) 
{
 UWORD i;
 UBYTE err; 

 //--- clock setup in PEE mode -------------
 //SetPEEmodeBUSCLK(0x01, 0x80, 0x01);        // 8MHz BUSCLK from 8 MHZ oscclk, PEE mode
 SetPEEmodeBUSCLK(0x58, 0x07, 0x00);        // 25MHz BUSCLK from 8 MHZ oscclk, PEE mode
 // ECLKCTL_NECLK = 0;                      // enable ECLK output (bus clock is visible on pin PS7) 

 //--- EEPROM setup ------------------------
 //EEPROM_Init(0x07);                          // for 8MHz BUSCLK
 EEPROM_Init(0x18);                          // for 25MHz BUSCLK


  
 for(i=EEPROM_START;i<EEPROM_END;i+=EEPROM_SECTOR) 
  { 
    buf[0] = temp++;
    buf[1] = temp++;
    
    err = EEPROM_Erase_Sector(i);
    if(err) asm nop;                       // process error
    err = EEPROM_Program(i, buf, 2);
    if(err) asm nop;                       // process error
  }
 

  
 for(;;) 
  { asm nop;
  }
}
//==============================================================================
