/*******************************************************************************
* Freescale Semiconductor Inc.
* (c) Copyright 2012 Freescale Semiconductor, Inc.
* ALL RIGHTS RESERVED.
********************************************************************************
Services performed by FREESCALE in this matter are performed AS IS and without 
any warranty. CUSTOMER retains the final decision relative to the total design 
and functionality of the end product. FREESCALE neither guarantees nor will be 
held liable by CUSTOMER for the success of this project.
FREESCALE DISCLAIMS ALL WARRANTIES, EXPRESSED, IMPLIED OR STATUTORY INCLUDING, 
BUT NOT LIMITED TO, IMPLIED WARRANTY OF MERCHANTABILITY OR FITNESS FOR 
A PARTICULAR PURPOSE ON ANY HARDWARE, SOFTWARE OR ADVISE SUPPLIED TO THE PROJECT
BY FREESCALE, AND OR NAY PRODUCT RESULTING FROM FREESCALE SERVICES. IN NO EVENT
SHALL FREESCALE BE LIABLE FOR INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF 
THIS AGREEMENT.

CUSTOMER agrees to hold FREESCALE harmless against any and all claims demands or 
actions by anyone on account of any damage, or injury, whether commercial, 
contractual, or tortuous, rising directly or indirectly as a result of an advise
or assistance supplied CUSTOMER in connection with product, services or goods 
supplied under this Agreement.
********************************************************************************
* File      main.c
* Owner     R62780
* Version   1.0
* Date      Feb-13-2012
* Classification   General Business Information
* Brief     Flash security example
********************************************************************************
* Detailed Description:
*
* The code secures the MCU after the first load and reset.
* It shows current security status (secured + backdoor)
* Then it uses backdoor key to unsecure the MCU
* It shows new security status (secured + backdoor)
* It uses SCI chanel (9600bps) to show security status, for exaple, by means of 
* Hyperterminal in the PC
* The code presents how to unsecure the device (backdoor key is used directly),
  but it is not presented how to receive backdoor key in real application.
  It can be done by SCI, CAN, SPI, ... Then presented routine should be used to
  unsecure the device.
*
* Notice that we have to add following command to "preload" command file 
  (P&E_Multilink_CyclonePro_Preload.cmd file in you project folder if you use
  PE Multilink):
  FLASH NOUNSECURE
* If we want to test security the BDM has to be disconnected. 
  (Unsecure the device by CodeWarrior if necessary, load the code, disconnect 
  BDM, press reset button or switch power button)
  Because the BDM is disconnected this example uses SCI to visualize what 
  is done.
* New code download can be done only after the MCU is unsecured by CodeWarrior 
  or another unsecure program because this code unsecures the MCU only till new
  reset. It does not changes security bytes in the FLASH at address 0xFF00
* 
*      
* tested on: TWR-S12G128
* OSCCLK = 8MHz oscillator, BUSCLK = 24MHz set by means of PLL in PEE mode
* Reference to documentation: MC9S12GRMV1 Rev.1.06
********************************************************************************
Revision History:
Version  Date         Author  Description of Changes
1.0      NOV-28-2011  R62780  Initial version
*******************************************************************************/
                         
#include <hidef.h>       /* common defines and macros       */
#include "derivative.h"  /* derivative-specific definitions */
#include "comdef.h"      /* my common definitions           */
#include "homer.h"       /* homer simpson                   */
#include <stdio.h>
/*******************************************************************************
* Local function prototypes
*******************************************************************************/

static void SetPEEmodeBUSCLK(UBYTE _synr, UBYTE _refdv, UBYTE _postdiv);
static void FLASH_checkSecurity(void);
static void SCI_Init(void);
static void SCI_sendText(SBYTE *str);
static void Delay(UWORD ii, UWORD jj);

//--------------------------------
//PFLASH Send_Command function
//--------------------------------
// this function is stored in RAM memory like variable. It is used to be able to
// perform flash command on any flash block. The code will be executed in RAM
// in C language:
//  {
//    FSTAT_CCIF = 1;         //launch command
//    while(FSTAT_CCIF == 0); //wait for done
//  }
static unsigned char Send_Command[]=
{
  0x1C, 0x01, 0x06, 0x80, 0x1F, 0x01, 0x06, 0x80, 0xFB, 0x3D
};


/*******************************************************************************
* Constants
*******************************************************************************/

//--------------------------------
//set and enable backdoor key 0xAAAA, 0xAAAA, 0xAAAA, 0xAAAA 
//and secure the flash
//--------------------------------
const unsigned char flash_array[] @0xFF00 = 
{
 0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xBD
};

/*******************************************************************************
* Local module variables
*******************************************************************************/

static   SBYTE txt[255];
volatile UBYTE err = 0;

/*******************************************************************************
Function Name : SCI_Init
Engineer      : R62780
Date          : APR/14/2011
Parameters    : NONE
Modifies      : NONE
Returns       : NONE
Notes         : SCI init at 9600bps @ 32MHz BUSCLK
Issues        : NONE
*******************************************************************************/
void  SCI_Init(void)
{
 //----- BR = BUSCLK / (16*Baud rate) = 24000000 / (16*9600) = 208 = 0x9C -----
                     //------------------------------------------------------
   SCI0BD  = 0x9C;   // prescaller value (depends on module clock frequency) 
                     // prescaller = 0xD0   9600Bd  (32MHz bus clock is set)
                     //------------------------------------------------------
   SCI0CR1 = 0x44;   // Loops sciswai rsrc   m    wake  ilt   pe    pt       
                     // 0-LOOPS=RSRC=0 => 1 Single-wire mode with Rx input connected to TXD
                     // 1-SCI dis. in wait mode                              
                     // 0-if RSRC=1-RxTx intern. connected if loops=1                  
                     // 0-start+8 data+1 stop bit                          
                     // 0-IDLE mark wake up                                  
                     // 1-idle char. bit count begins after stop bit         
                     // 0-parity fction disabled                             
                     // 0-odd parity                                       
                     //------------------------------------------------------
   SCI0CR2 = 0x08;   // tie   tcie    rie  ilie   te    re   rwu   sbk       
                     // 0-Tx empty intr disabled                             
                     // 0-Tx complete intr disabled                          
                     // 0-Rx full + OverRun interrupts
                     // 0-idle line intr disabled                            
                     // 1-transmitter enabled
                     // 0-receiver disabled
                     // 0-receiver normal operation                                   
                     // 0-no break characters                                
                     //------------------------------------------------------
}
/*******************************************************************************
Function Name : SCI_sendText
Engineer      : R62780
Date          : FEB/13/2011
Parameters    : NONE
Modifies      : NONE
Returns       : NONE
Notes         : send \0 terminated text out through SCI in polling mode
Issues        : NONE
*******************************************************************************/
void SCI_sendText(SBYTE *str)
{
  UBYTE i = 0;

  while(str[i]!='\0')
    {
     while(!SCI0SR1_TC);          // write data to SCI (clear TC) 
     SCI0DRL = str[i];
     i++;
    }
  while(str[i]!='\0');            //  wait while the character is being sent
}
/*******************************************************************************
Function Name : PLL_LockIsr
Engineer      : R62780
Date          : NOV/28/2011
Parameters    : NONE
Modifies      : NONE
Returns       : NONE
Notes         : The interrupt function used (if necessary) if the PLL lock 
                status has changed
Issues        : NONE
*******************************************************************************/
#pragma CODE_SEG NON_BANKED
interrupt 28 void PLL_LockIsr(void)
{
  if(CPMUFLG_LOCK == 0)  {    /*do something here*/  }
  else                   {    /*do something here*/  }
}
#pragma CODE_SEG DEFAULT
/*******************************************************************************
Function Name : setPEEmodeBUSCLK
Engineer      : R62780
Date          : NOV/28/2011
Parameters    : NONE
Modifies      : NONE
Returns       : NONE
Notes         : Changes BUSCLK
Issues        : NONE
*******************************************************************************/
void SetPEEmodeBUSCLK(UBYTE _synr, UBYTE _refdv, UBYTE _postdiv)
{
 CPMUSYNR    = _synr;
 CPMUREFDIV  = _refdv;
 CPMUPOSTDIV = _postdiv;   
  
 CPMUOSC_OSCE = 1;                   // enable external oscillator OSCE
    
 while(!CPMUFLG_UPOSC)
  {
    CPMUARMCOP=0x55;CPMUARMCOP=0xAA; // kick the dog if enabled
  };    
 while(!CPMUFLG_LOCK)
  {
    CPMUARMCOP=0x55;CPMUARMCOP=0xAA; // kick the dog if enabled
  };    
 
  //--- select clocks --------------------
 
 CPMUCLKS = 0B10000000;     // bus=fPLL/2; COP is clocked from OSCCLK
 if(CPMUCLKS != 0B10000011) // After writing CPMUCLKS register, it is strongly
  {                         // recommended to read back CPMUCLKS register 
                            // to make sure that write of PLLSEL, RTIOSCSEL,
     asm nop;               // COPOSCSEL0 and COPOSCSEL1 was successful.
  }
 asm nop;
 //--------------------------------------
}
/*******************************************************************************
Function Name : Delay
Engineer      : R62780
Date          : Apr/14/2010
Parameters    : NONE
Modifies      : NONE
Returns       : NONE
Notes         : Short delay - hard loop.
Issues        : NONE
*******************************************************************************/
static void Delay(UWORD ii, UWORD jj)
{
  unsigned int i,j;    
  
  for( i = 0; i < ii; i++ )  
    for( j = 0; j < jj; j++) asm nop;        
}
/*******************************************************************************
Function Name : main
Engineer      : R62780
Date          : Apr/14/2010
Parameters    : NONE
Modifies      : NONE
Returns       : NONE
Notes         : Main function.
Issues        : NONE
*******************************************************************************/
void main(void) 
{
 //-------------------------------
 // set BUSCLK in PEE mode
 //-------------------------------
 SetPEEmodeBUSCLK(0x02, 0x80, 0x00); // 24MHz BUSCLK from 8 MHZ oscclk, PEE mode
 //-------------------------------
 // visualize BUSCLK at PBO pin
 //-------------------------------
 ECLKCTL_NECLK = 0;                  // BUSCLK visible at PB0
 //-------------------------------
 // enable an external SCI tran-
 // sceiver device MAX3387E at the
 // TWR-S12G128 board
 //-------------------------------
 // *((UBYTE*)0x277) = 0;            // PRR1_PRR1AN; AN10~15 routed to PAD10~15
 DDR0AD_DDR0AD7 = 1;                 // set AN15 as an output
 PT0AD_PT0AD7   = 1;                 // enable MAX3387E
 //-------------------------------
 // set SCI fro communication
 //-------------------------------
 SCI_Init();                         // 9600 bps @ 32MHz BUSCLK
 //-------------------------------
 // say hello to se activity
 //-------------------------------
 homer();                            // homer face through SCI
 SCI_sendText((SBYTE*)"\n\r\n\rHELLO, I AM ALIVE....\n\r\n\r");
 //-------------------------------
 // set FLASH clock
 //-------------------------------
 FCLKDIV = 0x17;                     // Got from MC9S12GRMV1 for BUSCLK = 24MHz
 //-------------------------------
 // visualize current security status
 //-------------------------------
 (void) sprintf(txt,"Initial Security Status:\n\r");
 SCI_sendText(txt);
 FLASH_checkSecurity(); 
 //-------------------------------
 // unsecure the device
 //-------------------------------
 // Verify Backdoor Access Key
 // The Verify Backdoor Access Key command releases security if user-supplied
 // keys match those stored in the Flash security bytes of the Flash 
 // configuration field.
 //
 // This part of the code can be called "Unsecure the MCU". It is up to the user
 // which way he takes to perform this part of the code. Whether some action on
 // button or the security key is sent to the MCU via the SCI and then this
 // routine is performed with received backdoor key data.
 
 // note this unsecure is valid only till next reset because value at 0xFF0F
 // is not changed in this code
 //-------------------------------
  while(FSTAT_CCIF == 0);            // wait if command in progress
  FSTAT = 0x30;                      // clear ACCERR and PVIOL    
  

  FCCOBIX = 0x00;                    // Verify Backdoor Access Key
  FCCOB = 0x0C00;
  
  FCCOBIX = 0x01;                    // Backdoor key was defined in the variable
  FCCOB = 0xAAAA;                    // "flash_array" above as
  FCCOBIX = 0x02;                    // 0xAAAA,0xAAAA,0xAAAA,0xAAAA
  FCCOB = 0xAAAA;
  FCCOBIX = 0x03;
  FCCOB = 0xAAAA;
  FCCOBIX = 0x04;
  FCCOB = 0xAAAA;
  
  asm JSR Send_Command;
 //-------------------------------
 // visualize new security status
 //-------------------------------
 (void) sprintf(txt,"\n\rNew Security Status:\n\r");
 SCI_sendText(txt);
 FLASH_checkSecurity(); 
 //-------------------------------
 // loop forever
 //-------------------------------
 for(;;) {}
 //-------------------------------
}
/*******************************************************************************
The Answer to the Ultimate Question of Life, the Universe, and Everything is: 42
(Douglas Adams - The Hitchhiker's Guide to the Galaxy)
*******************************************************************************/

static void FLASH_checkSecurity(void)
{
 //-------------------------------
 // send FSEC, address 0xFF0F, FSTAT and FERSTAT to serial line:
 //-------------------------------
 (void) sprintf(txt,"FSEC    is: 0x%02X =>\n\r", FSEC);
 SCI_sendText(txt);

 if((FSEC & 0xC0) == 0x80) 
    (void) sprintf(txt,"\t\t\tKEYEN =  10 (BKGDOOR ENABLED)\n\r");
 else               
    (void) sprintf(txt,"\t\t\tKEYEN <> 10 (BKGDOOR DISABLED)\n\r");
 
 SCI_sendText(txt);

 if((FSEC & 0x03) == 0x02) 
    (void) sprintf(txt,"\t\t\tSEC   =  10 (UNSECURED)\n\r");
 else               
    (void) sprintf(txt,"\t\t\tSEC   <> 10 (SECURED)\n\r");

 SCI_sendText(txt);
 
 (void) sprintf(txt,"0xFF0F  is: 0x%2X \n\r", *(unsigned char *)0xFF0F);
 SCI_sendText(txt);
 
 (void) sprintf(txt,"FSTAT   is: 0x%02X \n\r", FSTAT);
 SCI_sendText(txt);
 
 (void) sprintf(txt,"FERSTAT is: 0x%02X \n\r", FERSTAT);
 SCI_sendText(txt);
}