/*******************************************************************************
* Freescale Semiconductor Inc.
* (c) Copyright 2004-2005 Freescale Semiconductor, Inc.
* ALL RIGHTS RESERVED.
********************************************************************************
Services performed by FREESCALE in this matter are performed AS IS and without 
any warranty. CUSTOMER retains the final decision relative to the total design 
and functionality of the end product. FREESCALE neither guarantees nor will be 
held liable by CUSTOMER for the success of this project.
FREESCALE DISCLAIMS ALL WARRANTIES, EXPRESSED, IMPLIED OR STATUTORY INCLUDING, 
BUT NOT LIMITED TO, IMPLIED WARRANTY OF MERCHANTABILITY OR FITNESS FOR 
A PARTICULAR PURPOSE ON ANY HARDWARE, SOFTWARE OR ADVISE SUPPLIED TO THE PROJECT
BY FREESCALE, AND OR NAY PRODUCT RESULTING FROM FREESCALE SERVICES. IN NO EVENT
SHALL FREESCALE BE LIABLE FOR INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF 
THIS AGREEMENT.

CUSTOMER agrees to hold FREESCALE harmless against any and all claims demands or 
actions by anyone on account of any damage, or injury, whether commercial, 
contractual, or tortuous, rising directly or indirectly as a result of an advise
or assistance supplied CUSTOMER in connection with product, services or goods 
supplied under this Agreement.
********************************************************************************
* File      main.c
* Owner     R62780
* Version   1.0
* Date      Feb-09-2012
* Classification   General Business Information
* Brief     Flash Erase/Write
********************************************************************************
* Detailed Description:
*
* The example present writing to the FLASH,
* The page 08 is used to be E/W so the line:
*      PAGE_08       = READ_ONLY   0x088000 TO 0x08BFFF;
* is removed from prm file to be not used bu compiler for code.
*
* Global address MUST be used for writing the flash. Logical address space
*      0x088000 TO 0x08BFFF corresponds to global address space 0x20000~0x23FFF
*
* Note that reding by means of global address is not possible. In this case only
* logical address can be used.
*
* The space 0x088000 TO 0x08BFFF is removed from memory space to be not used by
* compiler for code placement. See all lines labeled with text r62780 in the prm
* file to see changes
*
* Only global address is accepted for writing. 
* Reading is possible from ppage or local addresses only.
*     
* - tested on: TWR-S12G128
* - OSCCLK = 8MHz oscillator, BUSCLK = 24MHz set by means of PLL in PEE mode
* - Reference to documentation: MC9S12GRMV1 Rev.1.01
********************************************************************************
Revision History:
Version  Date         Author  Description of Changes
1.0      Feb-09-2012  R62780  Initial version
*******************************************************************************/
                         
#include <hidef.h>       /* common defines and macros       */
#include "derivative.h"  /* derivative-specific definitions */
#include "comdef.h"      /* my common definitions           */
#include "flash_g128.h"
/*******************************************************************************
* Local function prototypes
*******************************************************************************/
static void SetPEEmodeBUSCLK(UBYTE _synr, UBYTE _refdv, UBYTE _postdiv);
/*******************************************************************************
* Local module variables
*******************************************************************************/
static UWORD addr;
static ULONG addrl;
static UWORD buffer[4];   
static volatile UBYTE err;  
/*******************************************************************************
Function Name : PLL_LockIsr
Engineer      : R62780
Date          : NOV/28/2011
Parameters    : NONE
Modifies      : NONE
Returns       : NONE
Notes         : The interrupt function used (if necessary) if the PLL lock 
                status has changed
Issues        : NONE
*******************************************************************************/
#pragma CODE_SEG NON_BANKED
interrupt 28 void PLL_LockIsr(void)
{
  if(CPMUFLG_LOCK == 0)  {    /*do something here*/  }
  else                   {    /*do something here*/  }
}
#pragma CODE_SEG DEFAULT
/*******************************************************************************
Function Name : setPEEmodeBUSCLK
Engineer      : R62780
Date          : NOV/28/2011
Parameters    : NONE
Modifies      : NONE
Returns       : NONE
Notes         : Changes BUSCLK
Issues        : NONE
*******************************************************************************/
void SetPEEmodeBUSCLK(UBYTE _synr, UBYTE _refdv, UBYTE _postdiv)
{
 CPMUSYNR    = _synr;
 CPMUREFDIV  = _refdv;
 CPMUPOSTDIV = _postdiv;   
  
 CPMUOSC_OSCE = 1; //enable external oscillator OSCE
    
 while(!CPMUFLG_UPOSC)
  {
    CPMUARMCOP=0x55;CPMUARMCOP=0xAA; // kick the dog if enabled
  };    
 while(!CPMUFLG_LOCK)
  {
    CPMUARMCOP=0x55;CPMUARMCOP=0xAA; // kick the dog if enabled
  };    
 
  //--- select clocks --------------------
 
 CPMUCLKS = 0B10000000;     // bus=fPLL/2; COP is clocked from OSCCLK
 if(CPMUCLKS != 0B10000011) // After writing CPMUCLKS register, it is strongly
  {                         // recommended to read back CPMUCLKS register 
                            // to make sure that write of PLLSEL, RTIOSCSEL,
     asm nop;               // COPOSCSEL0 and COPOSCSEL1 was successful.
  }
 asm nop;
 //--------------------------------------
}
/*******************************************************************************
Function Name : main
Engineer      : R62780
Date          : Feb/02/2012
Parameters    : NONE
Modifies      : NONE
Returns       : NONE
Notes         : Main function.
Issues        : NONE
*******************************************************************************/
void main(void) 
{
 UWORD data=1;
 
 //.........................................
 SetPEEmodeBUSCLK(0x02, 0x80, 0x00); // 24MHz BUSCLK from 8 MHZ oscclk, PEE mode
 //.........................................
 ECLKCTL_NECLK = 0; // BUSCLK visible at PB0
 //.........................................
 DDRAB = 0xFFFF;    // an output ports only for debug purposes

 //.........................................
 //fill data buffer
 //.........................................

 buffer[0] = 0xAAAA;
 buffer[1] = 0xBBBB;
 buffer[2] = 0xCCCC;
 buffer[3] = 0xDDDD;

 //.........................................
 // init flash clock
 //.........................................

 PFLASH_Init(0x17);// 24MHz busclk

 //.........................................
 // Functions accept global address only.
 // Local and logical addresses can be converted to global address by 
 // "Local2Global" function the size of sectors is 512 bytes = 64 phrases
 //.........................................
 // check given flash is erased sector by sector, if it does not erase it ---
 // both following address accesses can be used 
 // ppage address of sector 0x088000 ~ 0x08BFFF
 // global address 0x20000 ~ 0x23FFF
 //.........................................

 // Note:
 // In order to see changes in the memory during debugging you should adjust
 // memory setup in the debugger:
 // menu/target(Multilink, Softec,...) -> Debugging Memory Map -> select
 // corresponding part of memory -> press Modify/Details -> select
 // "refresh memory when halting"

 
 for(addrl=0x088000UL; addrl<=0x08BFFFUL; addrl+=512) // entire ppage
   {
    if(PFLASH_Erase_Verify_Section(Local2Global(addrl),64) == NON_ERASED)
     {
        err = PFLASH_Erase_Sector(Local2Global(addrl));
     }
   }

 //.........................................
 // write the flash
 //.........................................

 for(addrl=0x088000UL; addrl<=0x08BFFFUL; addrl+=8)
   {
     err = PFLASH_Program(Local2Global(addrl), buffer); 
   }

 //.........................................
 // read the flash
 //.........................................

 for(addrl=0x088000UL; addrl<=0x08BFFFUL; addrl+=2)
   {
     PORTAB = *((UWORD *far) addrl); 
     //if you want to use long addresse you have to set PPAGE for paging:
     //Alt F7->Compiler for HC12->Option->Code Generation-> select PPAGE register
     //for paging and in the cell which appears below write 0x15
     // (the text -CpPPAGE=0x15 should appear in the compiler line arguments 
     //after this setup)
     
   }
  
 //.........................................
 //do the same using global address
 //.........................................

 for(addrl=0x020000UL; addrl<=0x023FFFUL; addrl+=512)
   {
    if(PFLASH_Erase_Verify_Section(addrl,64) == NON_ERASED)
     {
        err = PFLASH_Erase_Sector(addrl);
     }
   }
 
 for(addrl=0x020000UL; addrl<=0x023FFFUL; addrl+=8)
   {
     err = PFLASH_Program(addrl, buffer); 
   }
 
 // reading by global address is not possible so ppage or local address must be used
 for(addrl=0x088000UL; addrl<=0x08BFFFUL; addr+=2)
   {
     PORTAB = *((UWORD*) addr); 
   }
 
 //.........................................
 // wait forever
 //.........................................
 for(;;) {} 
 //.........................................

}
/*******************************************************************************
The Answer to the Ultimate Question of Life, the Universe, and Everything is: 42
(Douglas Adams - The Hitchhiker's Guide to the Galaxy)
*******************************************************************************/




























